==> simple-repository-1.6.0.tgz: package/lib/SimpleDynamoRepository.ts <==
import { DynamoDBClient } from '@aws-sdk/client-dynamodb'
import {
  DeleteCommand,
  DynamoDBDocumentClient,
  QueryCommand,
  ScanCommand,
  GetCommand,
  PutCommand,
  QueryCommandInput,
  ScanCommandInput,
} from '@aws-sdk/lib-dynamodb'
import { SimpleRepositoryNotFoundError } from './SimpleRepositoryNotFoundError'

interface SimpleDocumentType {
  id: string
}

/**
 * A simple repository for DynamoDB
 * @template DocumentType the type of the document
 * example:
 * ```ts
 * type User {
 *  id: string
 *  name: string
 * }
 * class UserRepository extends SimpleDynamoRepository<User> {}
 * const userRepository = new UserRepository(new DynamoDBClient({ region: 'eu-central-1' }), 'users')
 * ```
 */
export abstract class SimpleDynamoRepository<DocumentType extends SimpleDocumentType> {
  private readonly client: DynamoDBDocumentClient
  private readonly tableName: string

  constructor(client: DynamoDBClient, tableName: string) {
    this.client = DynamoDBDocumentClient.from(client, {
      marshallOptions: { removeUndefinedValues: false },
    })
    this.tableName = tableName
  }

  /**
   * Find a document by id
   * @param id
   * @throws DocumentNotFoundError if the document does not exist
   * @returns the document
   */
  async findOne(id: string): Promise<DocumentType> {
    const doc = await this.client.send(new GetCommand({ Key: { id }, TableName: this.tableName }))

    if (!doc.Item) throw new SimpleRepositoryNotFoundError(id)

    return doc.Item as DocumentType
  }

  /**
   * Find a document by the global secondary index
   * @param key the name of the secondary index
   * @param value the value of the secondary index
   * @returns all documents with the given secondary index
   */
  async findBy(secondaryKey: string, value: string): Promise<DocumentType[]> {
    const queryCommandInput: QueryCommandInput = {
      TableName: this.tableName,
      IndexName: secondaryKey,
      KeyConditionExpression: '#key = :value',
      ExpressionAttributeNames: { '#key': secondaryKey },
      ExpressionAttributeValues: { ':value': value },
    }
    const docs = await this.client.send(new QueryCommand(queryCommandInput))

    return docs.Items as DocumentType[]
  }

  /**
   * Find the first document by the global secondary index
   * @param key the name of the secondary index
   * @param value the value of the secondary index
   * @returns the first document with the given secondary index
   */
  async findOneBy(secondaryKey: string, value: string): Promise<DocumentType> {
    const queryCommandInput: QueryCommandInput = {
      TableName: this.tableName,
      IndexName: secondaryKey,
      KeyConditionExpression: '#key = :value',
      ExpressionAttributeNames: { '#key': secondaryKey },
      ExpressionAttributeValues: { ':value': value },
    }
    const docs = await this.client.send(new QueryCommand(queryCommandInput))

    if (docs && docs.Items && docs.Items.length > 0) return docs.Items[0] as DocumentType

    throw new SimpleRepositoryNotFoundError()
  }

  /**
   * Save a document
   * @param doc
   * @returns the saved document
   */
  async save(doc: DocumentType): Promise<DocumentType> {
    await this.client.send(
      new PutCommand({
        Item: doc as Record<string, any>,
        TableName: this.tableName,
      })
    )

    return doc
  }

  /**
   * Delete a document by id
   * @param id
   * @throws DocumentNotFoundError if the document does not exist
   * @returns void
   */
  async delete(id: string): Promise<void> {
    try {
      await this.client.send(
        new DeleteCommand({
          Key: { id },
          TableName: this.tableName,
          ConditionExpression: 'attribute_exists(id)',
        })
      )
    } catch (error: any) {
      if (error.name === 'ConditionalCheckFailedException')
        throw new SimpleRepositoryNotFoundError(id)
      else throw error
    }
  }

  /**
   * Find all documents in the table
   * @param limit
   * @param lastEvaluatedId the id of the last document in the previous page
   * @returns all documents in the table
   * example:
   * ```ts
   * let lastEvaluatedId: string | undefined = undefined
   * do {
   *   const users: User[] = await userRepository.findAll(10, lastEvaluatedId)
   *   lastEvaluatedId = users[users.length - 1]?.id || undefined
   * } while (lastEvaluatedId)
   * ```
   */
  async findAll(limit = 1000, lastEvaluatedId?: string): Promise<DocumentType[]> {
    const scanCommandInput: ScanCommandInput = {
      TableName: this.tableName,
      Limit: limit,
      ExclusiveStartKey: { id: lastEvaluatedId },
    }

    lastEvaluatedId == undefined && delete scanCommandInput.ExclusiveStartKey

    const docs = await this.client.send(new ScanCommand(scanCommandInput))

    return docs.Items as DocumentType[]
  }

  /**
   * Count all documents in the table
   * @returns the number of documents in the table
   */
  async count(): Promise<number> {
    const docs = await this.client.send(
      new ScanCommand({ TableName: this.tableName, Select: 'COUNT' })
    )

    return docs.Count || 0
  }

  /**
   * Find by query
   * @param queryCommandInput
   * @returns all documents that match the query
   */
  async queryBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType[]> {
    const docs = await this.client.send(
      new QueryCommand({
        ...queryCommandInput,
        TableName: this.tableName,
      })
    )

    return docs.Items as DocumentType[]
  }

  /**
   * Find one by query
   * @param queryCommandInput
   * @returns the first document that matches the query
   */
  async queryOneBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType> {
    const docs = await this.client.send(
      new QueryCommand({
        ...queryCommandInput,
        TableName: this.tableName,
      })
    )

    if (docs && docs.Items && docs.Items.length > 0) return docs.Items[0] as DocumentType

    throw new SimpleRepositoryNotFoundError()
  }
}

==> simple-repository-1.7.0.tgz: package/lib/SimpleDynamoRepository.ts <==
import { DynamoDBClient } from '@aws-sdk/client-dynamodb'
import {
  DeleteCommand,
  DynamoDBDocumentClient,
  QueryCommand,
  ScanCommand,
  GetCommand,
  PutCommand,
  QueryCommandInput,
  ScanCommandInput,
} from '@aws-sdk/lib-dynamodb'
import { SimpleRepositoryNotFoundError } from './SimpleRepositoryNotFoundError'

interface SimpleDocumentType {
  id: string
}

/**
 * A simple repository for DynamoDB
 * @template DocumentType the type of the document
 * example:
 * ```ts
 * type User {
 *  id: string
 *  name: string
 * }
 * class UserRepository extends SimpleDynamoRepository<User> {}
 * const userRepository = new UserRepository(new DynamoDBClient({ region: 'eu-central-1' }), 'users')
 * ```
 */
export abstract class SimpleDynamoRepository<DocumentType extends SimpleDocumentType> {
  protected readonly client: DynamoDBDocumentClient
  private readonly tableName: string

  constructor(client: DynamoDBClient, tableName: string) {
    this.client = DynamoDBDocumentClient.from(client, {
      marshallOptions: { removeUndefinedValues: false },
    })
    this.tableName = tableName
  }

  /**
   * Find a document by id
   * @param id
   * @throws DocumentNotFoundError if the document does not exist
   * @returns the document
   */
  async findOne(id: string): Promise<DocumentType> {
    const doc = await this.client.send(new GetCommand({ Key: { id }, TableName: this.tableName }))

    if (!doc.Item) throw new SimpleRepositoryNotFoundError(id)

    return doc.Item as DocumentType
  }

  /**
   * Find a document by the global secondary index
   * @param key the name of the secondary index
   * @param value the value of the secondary index
   * @returns all documents with the given secondary index
   */
  async findBy(secondaryKey: string, value: string): Promise<DocumentType[]> {
    const queryCommandInput: QueryCommandInput = {
      TableName: this.tableName,
      IndexName: secondaryKey,
      KeyConditionExpression: '#key = :value',
      ExpressionAttributeNames: { '#key': secondaryKey },
      ExpressionAttributeValues: { ':value': value },
    }
    const docs = await this.client.send(new QueryCommand(queryCommandInput))

    return docs.Items as DocumentType[]
  }

  /**
   * Find the first document by the global secondary index
   * @param key the name of the secondary index
   * @param value the value of the secondary index
   * @returns the first document with the given secondary index
   */
  async findOneBy(secondaryKey: string, value: string): Promise<DocumentType> {
    const queryCommandInput: QueryCommandInput = {
      TableName: this.tableName,
      IndexName: secondaryKey,
      KeyConditionExpression: '#key = :value',
      ExpressionAttributeNames: { '#key': secondaryKey },
      ExpressionAttributeValues: { ':value': value },
    }
    const docs = await this.client.send(new QueryCommand(queryCommandInput))

    if (docs && docs.Items && docs.Items.length > 0) return docs.Items[0] as DocumentType

    throw new SimpleRepositoryNotFoundError()
  }

  /**
   * Save a document
   * @param doc
   * @returns the saved document
   */
  async save(doc: DocumentType): Promise<DocumentType> {
    await this.client.send(
      new PutCommand({
        Item: doc as Record<string, any>,
        TableName: this.tableName,
      })
    )

    return doc
  }

  /**
   * Delete a document by id
   * @param id
   * @throws DocumentNotFoundError if the document does not exist
   * @returns void
   */
  async delete(id: string): Promise<void> {
    try {
      await this.client.send(
        new DeleteCommand({
          Key: { id },
          TableName: this.tableName,
          ConditionExpression: 'attribute_exists(id)',
        })
      )
    } catch (error: any) {
      if (error.name === 'ConditionalCheckFailedException')
        throw new SimpleRepositoryNotFoundError(id)
      else throw error
    }
  }

  /**
   * Find all documents in the table
   * @param limit
   * @param lastEvaluatedId the id of the last document in the previous page
   * @returns all documents in the table
   * example:
   * ```ts
   * let lastEvaluatedId: string | undefined = undefined
   * do {
   *   const users: User[] = await userRepository.findAll(10, lastEvaluatedId)
   *   lastEvaluatedId = users[users.length - 1]?.id || undefined
   * } while (lastEvaluatedId)
   * ```
   */
  async findAll(limit = 1000, lastEvaluatedId?: string): Promise<DocumentType[]> {
    const scanCommandInput: ScanCommandInput = {
      TableName: this.tableName,
      Limit: limit,
      ExclusiveStartKey: { id: lastEvaluatedId },
    }

    lastEvaluatedId == undefined && delete scanCommandInput.ExclusiveStartKey

    const docs = await this.client.send(new ScanCommand(scanCommandInput))

    return docs.Items as DocumentType[]
  }

  /**
   * Count all documents in the table
   * @returns the number of documents in the table
   */
  async count(): Promise<number> {
    const docs = await this.client.send(
      new ScanCommand({ TableName: this.tableName, Select: 'COUNT' })
    )

    return docs.Count || 0
  }

  /**
   * Find by query
   * @param queryCommandInput
   * @returns all documents that match the query
   */
  async queryBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType[]> {
    const docs = await this.client.send(
      new QueryCommand({
        ...queryCommandInput,
        TableName: this.tableName,
      })
    )

    return docs.Items as DocumentType[]
  }

  /**
   * Find one by query
   * @param queryCommandInput
   * @returns the first document that matches the query
   */
  async queryOneBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType> {
    const docs = await this.client.send(
      new QueryCommand({
        ...queryCommandInput,
        TableName: this.tableName,
      })
    )

    if (docs && docs.Items && docs.Items.length > 0) return docs.Items[0] as DocumentType

    throw new SimpleRepositoryNotFoundError()
  }
}
